"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _container = require("./container");

var _container2 = _interopRequireDefault(_container);

var _alert = require("./alert");

var _alert2 = _interopRequireDefault(_alert);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var AlertList = function AlertList(_ref) {
	var position = _ref.position;
	var alerts = _ref.alerts;
	var onDismiss = _ref.onDismiss;

	var props = _objectWithoutProperties(_ref, ["position", "alerts", "onDismiss"]);

	return _react2.default.createElement(
		_container2.default,
		{ position: position },
		alerts.map(function (item) {
			var dismiss = onDismiss ? function () {
				return onDismiss(item);
			} : null;

			var message = item.message;

			var alertProps = _objectWithoutProperties(item, ["message"]);

			return _react2.default.createElement(
				_alert2.default,
				_extends({ key: item.id }, props, alertProps, { onDismiss: dismiss }),
				message
			);
		})
	);
};

var AlertListWithTimers = function (_Component) {
	_inherits(AlertListWithTimers, _Component);

	function AlertListWithTimers(props) {
		_classCallCheck(this, AlertListWithTimers);

		var _this = _possibleConstructorReturn(this, (AlertListWithTimers.__proto__ || Object.getPrototypeOf(AlertListWithTimers)).call(this, props));

		_this.timers = {};
		return _this;
	}

	_createClass(AlertListWithTimers, [{
		key: "componentDidMount",
		value: function componentDidMount() {
			if (this.props.alerts) {
				this.setupTimers(this.props.alerts, this.props.timeout, this.props.onDismiss, this.timers);
			}
		}
	}, {
		key: "componentWillReceiveProps",
		value: function componentWillReceiveProps(_ref2) {
			var alerts = _ref2.alerts;
			var timeout = _ref2.timeout;
			var onDismiss = _ref2.onDismiss;

			if (alerts) {
				this.setupTimers(alerts, timeout, onDismiss, this.timers);
			}
		}
	}, {
		key: "componentWillUnmount",
		value: function componentWillUnmount() {
			// need to clean up after ourselves
			this.setupTimers();
		}
	}, {
		key: "setupTimers",
		value: function setupTimers(alerts, timeout, onDismiss, timers) {
			var _this2 = this;

			if (!timeout || !onDismiss) {
				// clear any timers we currently have
				for (var alertId in timers) {
					var timer = timers[alertId];
					clearTimeout(timer);
					delete timers[alertId];
				}
			} else {
				// add timers we don't have yet for any new alerts
				alerts.forEach(function (alert) {
					if (!timers[alert.id]) {
						// if there is no timer setup yet for this alert, set one up

						timers[alert.id] = setTimeout(_this2.dismissAlert.bind(_this2, timers, onDismiss, alert), timeout + _container.ENTER_TIMEOUT + _container.EXIT_TIMEOUT);
					}
				});

				// clear any timers we have for alerts that are no longer here

				var _loop = function _loop(_alertId) {
					var exists = alerts.filter(function (a) {
						return a.id == _alertId;
					}).length > 0;

					if (!exists) {
						var _timer = timers[_alertId];
						clearTimeout(_timer);
						delete timers[_alertId];
					}
				};

				for (var _alertId in timers) {
					_loop(_alertId);
				}
			}
		}
	}, {
		key: "dismissAlert",
		value: function dismissAlert(timers, onDismiss, alert) {
			// clear the timer if it hasn't fired yet
			clearTimeout(timers[alert.id]);

			// we don't need to keep track of any timers for this alert anymore
			delete timers[alert.id];

			// actually dismiss the alert
			onDismiss(alert);
		}
	}, {
		key: "render",
		value: function render() {
			var onDismiss = this.props.onDismiss ? this.dismissAlert.bind(this, this.timers, this.props.onDismiss) : null;
			return _react2.default.createElement(AlertList, _extends({}, this.props, { onDismiss: onDismiss }));
		}
	}]);

	return AlertListWithTimers;
}(_react.Component);

AlertListWithTimers.propTypes = _extends({}, _container.PropTypes, {
	alerts: _react.PropTypes.arrayOf(_react.PropTypes.shape({
		id: _react.PropTypes.any.isRequired,
		type: _alert.PropTypes.type,
		headline: _react.PropTypes.string,
		message: _react.PropTypes.string
	})).isRequired,
	onDismiss: _react.PropTypes.func,
	timeout: _react.PropTypes.number
});

exports.default = AlertListWithTimers;